-- =====================================================
--
--   SyncSRAM interface, address counter, sample counter
--
-- =====================================================
--
-- This is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2, or (at your option)
-- any later version.
--
-- This software is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this package; see the file COPYING.  If not, write to
-- the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-- Boston, MA 02111-1307, USA.


-- $Id: sram.vhd,v 1.8  2010/12/01 14:10:23  tinman Exp $
--
-- $Log: sram.vhd,v $
-- Revision 1.8  2010/12/01 14:10:23  tinman
-- NOTE : THIS IS NOT OFFICIAL MINILA BUILD
-- check http://www.mikrocontroller.net/articles/Minila_Version_MockUp
-- added support for 512k SRAM
-- status LED changed
-- constraints NOT compatible with all older versions!

-- Revision 1.7.2.1  2006/08/30 09:39:44  jsmrz
-- Returned sample_cnt_top_int to sample counter enable.
--
-- Revision 1.7  2006/08/25 12:05:06  jsmrz
-- Added trigger edge detectors,
-- optimized timing.
--
-- Revision 1.6  2005/07/19 10:18:42  jsmrz
-- no changes
--
-- Revision 1.5.2.3  2005/04/21 13:12:21  jsmrz
-- sram_clk renamed to clk
-- added license info
--
-- Revision 1.5.2.2  2005/04/21 12:55:15  jsmrz
-- sample_cnt_wm comparation value changed
-- english commentary
--
-- Revision 1.5.2.1  2005/04/20 14:53:57  jsmrz
-- signal sample_cnt_pretrig renamed to sample_cnt_wm
-- sample_cnt_wm condition fixed
--
-- Revision 1.5  2005/04/20 12:17:00  jsmrz
-- posttrigger counter changed to pretrigger
-- burst-counter removed
-- SRAM CE now always active
--
-- Revision 1.4  2005/02/07 16:38:52  jsmrz
-- Initial revision
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity sram is
   port
   (
      clr		: in std_logic;				-- 1-> counter resets
      clk		: in std_logic;				-- clock input

      ce_l		: out std_logic;			-- SRAM chip enable
      oe_l		: out std_logic;			-- SRAM output enable
      we_l		: out std_logic;			-- SRAM write enable
      adsc_l		: out std_logic;			-- SRAM address strobe
      sram_write	: in std_logic;				-- write to SRAM enabled
      adr_cnt_en	: in std_logic;				-- address counter enable
      adr_cnt		: out std_logic_vector(18 downto 0);	-- SRAM address

      psize_reg		: in std_logic_vector(5 downto 0);	-- pre/posttrigger length
      sample_cnt_en	: in std_logic;				-- sample counter enable
      sample_cnt_wm	: out std_logic;			-- sample counter wm hit
      sample_cnt_top	: out std_logic				-- sample counter top hit
   );
end entity sram;

architecture rtl of sram is

   signal adr_cnt_int		: std_logic_vector(18 downto 0) := (others => '0');-- SRAM address counter
   signal sample_cnt_int	: std_logic_vector(18 downto 0);-- pretrigger/postrigger counter
   signal sample_cnt_top_int	: std_logic;			-- sample counter top hit (internal)

begin

   -- 1 ns delays added to avoid hold-time violation errors in RTL simulation

   we_l <= not(sram_write) after 1 ns;
   oe_l <= sram_write after 1 ns;
   ce_l <= '0';
   adsc_l <= '0';

  
   -- address counter
   process(clk)
   begin
      if clk'event and clk = '1' then
         if adr_cnt_en = '1' then
            adr_cnt_int <= adr_cnt_int + conv_std_logic_vector(1, 19);
	 end if;
      end if;
   end process;

   adr_cnt <= adr_cnt_int after 1 ns;
   
   -- sample counter (pretrigger + posttrigger)
   process(clr, clk)
   begin
      if clr = '1' then
         sample_cnt_int <= (others => '0');
      elsif clk'event and clk = '1' then
         if (sample_cnt_en = '1') and (sample_cnt_top_int = '0') then
            sample_cnt_int <= sample_cnt_int + conv_std_logic_vector(1, 19);
	 end if;
      end if;
   end process;
   
   -- sample counter watermark registered to increase the speed of the design
   process(clk)
   begin
      if clk'event and clk = '1' then
         if (sample_cnt_int(18 downto 13) = psize_reg) and (sample_cnt_int(12 downto 0) ="1111111111101") then
            sample_cnt_wm <= '1';
	 else
	    sample_cnt_wm <= '0';
	 end if;
      end if;
   end process;
			 
   sample_cnt_top_int <= '1' when sample_cnt_int = "1111111111111111010" else
		         '0';

   sample_cnt_top <= sample_cnt_top_int;
 
end rtl;   
